'''
WC 2 - Introduction to Python

This weekly challenge was to generate a robust quadratic solver in python.
Key things to keep in mind:

    1. It was to be turned in as a .py file.
    2. It was to be written as a function.
    3. The function could be either a single "robust" one, or two; however,
       in the end one of them should have returned two roots (including the
       handling of imaginary ones).
'''


def quadratic_solver_1(a, b, c=0):
    '''
    To get the roots, we will use the built in python complex data type.
    A common mistake is to not wrap 2.0 * a in parenthesis!  That means that,
    by the rules of PEMDAS, your math was wrong!
    '''
    root_1 = (-b + complex(b**2 - 4 * a * c)**0.5) / (2.0 * a)
    root_2 = (-b - complex(b**2 - 4 * a * c)**0.5) / (2.0 * a)
    return root_1, root_2


def quadratic_solver_2(a, b, c=0):
    '''
    This is an alternative approach that avoids using imaginary numbers
    themselves by splitting up the maths and returning strings instead.
    '''
    # Calculate the determinant, and set a flag if we have an imaginary root
    det = b**2 - 4.0 * a * c
    is_imaginary = det < 0
    det = abs(det)**0.5

    # Calculate the different parts of the root
    part_1 = -b / (2.0 * a)
    part_2 = det / (2.0 * a)

    # Return the results
    if is_imaginary:
        root_1 = "%.2f + %.2fi" % (part_1, part_2)
        root_2 = "%.2f - %.2fi" % (part_1, part_2)
        return root_1, root_2
    else:
        root_1 = "%.2f" % (part_1 + part_2)
        root_2 = "%.2f" % (part_1 - part_2)
        return root_1, root_2


if __name__ == "__main__":
    print("Using Approach 1:")
    print("Roots of a, b, c = (3, 2, 0) are %s"
          % str(quadratic_solver_1(3, 2, c=0)))
    print("Roots of a, b, c = (3, 2, 1) are %s"
          % str(quadratic_solver_1(3, 2, c=1)))

    print("Using Approach 2:")
    print("Roots of a, b, c = (3, 2, 0) are %s and %s"
          % quadratic_solver_2(3, 2, c=0))
    print("Roots of a, b, c = (3, 2, 1) are %s and %s"
          % quadratic_solver_2(3, 2, c=1))
